@@ -3,6 +3,11 @@
 
 source $TOP/tests/common
 
+if ! losetup --help | grep -q 'partscan'; then
+	_not_run "losetup --partscan not available"
+	exit 0
+fi
+
 check_prereq mkfs.btrfs
 
 setup_root_helper
@@ -18,7 +23,7 @@ base=$(basename $loopdev)
 # expect partitions named like loop0p1 etc
 for looppart in $(ls /dev/$base?*); do
 	run_check $SUDO_HELPER $TOP/mkfs.btrfs -f $looppart
-	run_check $TOP/btrfs inspect-internal dump-super $looppart
+	run_check $SUDO_HELPER $TOP/btrfs inspect-internal dump-super $looppart
 done
 
 # cleanup
